#include <stdio.h>
#include "bfctype.h" 


/* Extended index table is used to find the mapping index of a character [0-65535].

  If you can find a simple mathematical expression for index mapping, 
  you can use that, and undefine USE_EXTENDED_INDEX_TABLE.

  If you do not have such a mathematical expression, this index table is just for you.

  To get the index of character 'A', you need to calculate it like the following: */

/* a sample function to get the character index on the extended index table. */
USHORT GetExtendedIndex(USHORT glyph, const FONTHEADER_BFC *pFontHeader)
{
	if( pFontHeader == NULL )
	      return 0; 

	USHORT firstChar = pFontHeader->firstChar;
	USHORT lastChar  = pFontHeader->lastChar;

	if( glyph < firstChar || glyph > lastChar )
		return 0;  /* not found, return the first glyph index */

	USHORT index = 0;

	/* 1. use ascii_index_table */
	UCHAR *ascii_index_table = pFontHeader->ascii_index_table;
	if( ascii_index_table != NULL )
	{
		if( glyph <= 255 )
		{
			index = ascii_index_table[glyph];
			return index;
		}
	}

	/* 2. use extended_index_table */
	GLYPH_SET *pGlyphSet = pFontHeader->extended_index_table;
	if( pGlyphSet == NULL )
		return 0; 

	GLYPH_RANGE *pRanges = pGlyphSet->pRanges;
	if( pRanges == NULL )
		return 0; 

	USHORT nRanges = pGlyphSet->nRanges;
	for(USHORT i=0; i<nRanges; i++)
	{
		USHORT charLow = pRanges[i].charLow;
		USHORT nGlyphs = pRanges[i].nGlyphs;

		if( glyph < charLow+nGlyphs )            
		{
			index += (glyph - charLow); 
                return index;
		}
		else
		{
			index += nGlyphs;
		}
	}

	return 0;   /* not found, return the first glyph index */

}
